--[[============================================================================
main.lua
============================================================================]]--

local rs

-- Placeholder for the dialog
local dialog = nil

-- Placeholder to expose the ViewBuilder outside the show_dialog() function
local vb = nil

local pattern, line, step, track, column, tracktype, nv

--------------------------------------------------------------------------------
-- Track.track_index property
--------------------------------------------------------------------------------
local track_index_property = property(function(self)
  for index, track in ipairs(renoise.song().tracks) do
    if (rawequal(self, track)) then
      return index
    end
  end
end)
renoise.Track.track_index = track_index_property
renoise.GroupTrack.track_index = track_index_property

--------------------------------------------------------------------------------
-- GroupTrack.leaves_indexes property
--------------------------------------------------------------------------------
function find_leaves_indexes(self)
  local list = table.create()
  for _, t in ipairs(self.members) do
    if t.type == 1 then
      list:insert(t.track_index)
    elseif t.type == 4 then
      local sublist = find_leaves_indexes(t)
      for i = 1, #sublist do
        list:insert(sublist[i])
      end
    end
  end
  return list
end
local grouptrack_leaves_indexes_property = property(find_leaves_indexes)
renoise.GroupTrack.leaves_indexes = grouptrack_leaves_indexes_property

--------------------------------------------------------------------------------
-- Main functions
--------------------------------------------------------------------------------

local function pushback_delay_column(mm)
  -- if note column change delay_value in current note_column
  -- if instr track change delay_value in every note_column
  -- if group track change delay_value in every note_column in every sub track that's an instr track
  rs = renoise.song()
  nv = mm.int_value * 2
  pattern = rs.sequencer:pattern(rs.transport.edit_pos.sequence)
  line = rs.transport.edit_pos.line
  step = rs.transport.edit_step
  track = rs.selected_track_index
  tracktype = rs.selected_track.type
  column = rs.selected_note_column_index
  if step == 0 then
    if column >= 1 and column <= 12 then
      rs:track(track).delay_column_visible = true
      rs:pattern(pattern):track(track):line(line):note_column(column).delay_value = nv
    elseif tracktype == 1 then
      rs:track(track).delay_column_visible = true
      --for c = 1, rs:track(track).max_note_columns do
      for c = 1, rs:track(track).visible_note_columns do
        rs:pattern(pattern):track(track):line(line):note_column(c).delay_value = nv
      end
    elseif tracktype == 4 then
      -- NOTE: not yet fully implemented
      for _, t in pairs(rs:track(track).leaves_indexes) do
        rs:track(t).delay_column_visible = true
        for c = 1, rs:track(t).visible_note_columns do
          rs:pattern(pattern):track(t):line(line):note_column(c).delay_value = nv
        end
      end
    else
      renoise.app():show_status("dude u r in wrong track type")
    end
  else
    rs.transport.follow_player = false
    if column >= 1 and column <= 12 then -- instrument track
      rs:track(track).delay_column_visible = true
      local x = line % step
      if x < 1 then x = 1 end
      --while x > step do x = x - step end
      for l = x, rs:pattern(pattern).number_of_lines, step do
        rs:pattern(pattern):track(track):line(l):note_column(column).delay_value = nv
      end
    elseif tracktype == 1 then
      rs:track(track).delay_column_visible = true
      local x = line % step
      if x < 1 then x = 1 end
      for l = x, rs:pattern(pattern).number_of_lines, step do
        for c = 1, rs:track(track).max_note_columns do
          rs:pattern(pattern):track(track):line(l):note_column(c).delay_value = nv
        end
      end
    elseif tracktype == 4 then
      -- NOTE: not yet implemented at all
      for _, t in pairs(rs:track(track).leaves_indexes) do
        rs:track(t).delay_column_visible = true
        local x = line % step
        if x < 1 then x = 1 end
        for l = x, rs:pattern(pattern).number_of_lines, step do
          for c = 1, rs:track(t).visible_note_columns do
            rs:pattern(pattern):track(t):line(l):note_column(c).delay_value = nv
          end
        end
      end
    else
      renoise.app():show_status("dude u r in wrong track type")
    end
  end
end



--local function up()
  --vb.views.pushback.value = vb.views.pushback.value + 1
  --pushback_delay_column()
--end
--local function down(d)
  --vb.views.pushback.value = vb.views.pushback.value - 1
  --pushback_delay_column()
--end

--local function key_dialog(d,k)
  --if ( k.name == "up" ) then
    --up()
  --elseif ( k.name == "down" ) then
    --down()
  --else
    --return k
  --end
--end


--------------------------------------------------------------------------------
-- Key Binding
--------------------------------------------------------------------------------



--------------------------------------------------------------------------------
-- MIDI Mapping
--------------------------------------------------------------------------------

renoise.tool():add_midi_mapping {
  name = "Tools:Push back",
  invoke = function(mm) pushback_delay_column(mm) end
}




-- Reload the script whenever this file is saved. 
-- Additionally, execute the attached function.
_AUTO_RELOAD_DEBUG = function()
  --print(toint("FF"))
end

